module Filter where

import SimpleParser
import Gale
import Char
import List

------------------------
-- Filter implementation
------------------------

data Filter = FilterSub String | FilterAuthor String | FilterCategory Category  | 
    FilterKeyword String | FilterSender String | FilterNot Filter |
    FilterOr [Filter] | FilterAnd [Filter]
	    deriving(Eq)

parseFilter :: String -> Maybe Filter
parseFilter = parser (between spaces eof mp) where
    mp = many1 fp >>= \v -> case v of
	    [x] -> return x
	    xs -> return $ FilterAnd xs
    fp = nf <|> flt <|> fmap FilterSub (token string) 
    nf = char '!' >> fmap FilterNot fp
    string = qstring <|> many1 (noneOf " \t\n~'!()|;&")
    qstring = between (char '\'') (char '\'') $ many ((char '\'' >> char '\'' >> return '\'') <|> noneOf "'")
    flt = do
	char '~'
	c <- sat isAlpha
	option ':' (char ':')
	s <- string
	spaces
	return $ z c s
    z 'a' s = FilterAuthor s
    z 's' s = FilterSender s
    z 'c' s = FilterCategory (catParseNew s)
    z 'k' s = FilterKeyword s
    z _ s = FilterSub s


applyFilterStack :: [Filter] -> [(Int, Puff)] -> [(Int, Puff)]
applyFilterStack fs = concatMap f where
    f a@(_,p) | all (flip filterAp p) fs = [a]
    f _ = []


siglookup c (Unverifyable (Key n _):_) | c == n = True
siglookup c (Signed (Key n _):_) | c == n = True
siglookup c (_:xs) = siglookup c xs
siglookup _ [] = False



filterAp :: Filter -> Puff -> Bool
filterAp (FilterNot f) p = not $ filterAp f p
filterAp (FilterSender s) p = case getFragmentString p "message/sender" of
    (Just s') | s' == s -> True
    _ -> False
filterAp (FilterAuthor c) (Puff {signature = sig})  = siglookup c sig 
filterAp (FilterKeyword k) p = any (k `isPrefixOf`) $ getFragmentStrings p "message.keyword"
filterAp (FilterCategory c) (Puff {cats = cs}) = any (`subCategory` c) cs
filterAp (FilterSub s) p = case getFragmentString p "message/body" of
    Just b | any (s `isPrefixOf`) (tails b) ->  True
    _ -> False
filterAp (FilterAnd fs) p = all (`filterAp` p) fs
filterAp (FilterOr fs) p = any (`filterAp` p) fs

showFilter :: Filter -> String
showFilter (FilterSub s) = s
showFilter (FilterAuthor a) = "~a:" ++  a
showFilter (FilterSender s) = "~s:" ++ s
showFilter (FilterCategory c) = "~c:" ++ catShowNew c
showFilter (FilterKeyword c) = "~k:" ++ c
showFilter (FilterNot f) = "!" ++ showFilter f
showFilter (FilterAnd fs) = "(" ++ concat (intersperse " " (map showFilter fs)) ++ ")"
showFilter (FilterOr fs) = "(" ++ (concat $ intersperse " ; " (map showFilter fs)) ++ ")"
--showFilter _ = ""
